﻿/* Written by Brian Lagunas
 * Blog: http://brianlagunas.com
 * Twitter: @brianlagunas
 * Email: blagunas@infragistics.com 
 */
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using Infragistics.Windows.DockManager;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Prism.Regions.Behaviors;
using System.Windows.Data;

namespace Infrastructure.Prism
{
    public class TabGroupPaneRegionBehavior : RegionBehavior, IHostAwareRegionBehavior
    {
        public const string BehaviorKey = "TabGroupPaneRegionBehavior";

        /// <summary>
        /// Used to determine what views were injected and ContentPanes were generated for
        /// </summary>
        private static readonly DependencyProperty IsGeneratedProperty = DependencyProperty.RegisterAttached("IsGenerated", typeof(bool), typeof(TabGroupPaneRegionBehavior), null);

        TabGroupPane _hostControl;
        public System.Windows.DependencyObject HostControl
        {
            get { return _hostControl; }
            set { _hostControl = value as TabGroupPane; }
        }

        protected override void OnAttach()
        {
            //if it's databound we do not allow view injection
            if (_hostControl.ItemsSource != null)
                throw new InvalidOperationException("ItemsControl's ItemsSource property is not empty. This control is being associated with a region, but the control is already bound to something else. If you did not explicitly set the control's ItemSource property, this exception may be caused by a change in the value of the inherited RegionManager attached property.");

            SynchronizeItems();

            var dockManager = XamDockManager.GetDockManager(_hostControl);
            if (dockManager != null)
                dockManager.ActivePaneChanged += DockManager_ActivePaneChanged;

            Region.Views.CollectionChanged += Views_CollectionChanged;
        }

        void DockManager_ActivePaneChanged(object sender, RoutedPropertyChangedEventArgs<ContentPane> e)
        {
            if (e.OldValue != null)
            {
                var item = e.OldValue;

                //this first checks to see if we had any default views declared in XAML and that were not injected
                if (Region.Views.Contains(item) && Region.ActiveViews.Contains(item))
                {
                    Region.Deactivate(item);
                }
                else
                {
                    //now check to see if we have any views that were injected
                    var contentControl = item as ContentControl;
                    if (contentControl != null)
                    {
                        var injectedView = contentControl.Content;
                        if (Region.Views.Contains(injectedView) && Region.ActiveViews.Contains(injectedView))
                            Region.Deactivate(injectedView);
                    }
                }
            }

            if (e.NewValue != null)
            {
                var item = e.NewValue;

                //this first checks to see if we had any default views declared in XAML and that were not injected
                if (Region.Views.Contains(item) && !this.Region.ActiveViews.Contains(item))
                {
                    Region.Activate(item);
                }
                else
                {
                    //now check to see if we have ay views that were injected
                    var contentControl = item as ContentControl;
                    if (contentControl != null)
                    {
                        var injectedView = contentControl.Content;
                        if (Region.Views.Contains(injectedView) && !this.Region.ActiveViews.Contains(injectedView))
                            Region.Activate(injectedView);
                    }
                }
            }
        }

        /// <summary>
        /// Takes all the views that were declared in XAML manually and merges them with any views that were injected into the region.
        /// </summary>
        private void SynchronizeItems()
        {
            List<object> existingItems = new List<object>();
            if (_hostControl.Items.Count > 0)
            {
                foreach (object childItem in _hostControl.Items)
                {
                    existingItems.Add(childItem);
                }
            }

            foreach (object view in Region.Views)
            {
                var contentPane = PrepareContainerForItem(view);
                _hostControl.Items.Add(contentPane);
            }

            foreach (object existingItem in existingItems)
            {
                PrepareContainerForItem(existingItem);
                Region.Add(existingItem);
            }
        }

        void Views_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            //new views are being injected, so lets add them to the region
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                //we want to add them behind any previous views that may have been manually declare in XAML or injected
                int startIndex = e.NewStartingIndex;
                foreach (object newItem in e.NewItems)
                {
                    ContentPane contentPane = PrepareContainerForItem(newItem);
                    _hostControl.Items.Insert(startIndex++, contentPane);
                }
            }

            //We do not need to implement the NotifyCollectionChangedAction.Remove functionality becase we are setting the ContentPane.CloseAction = PaneCloseAction.RemovePane.
            //This places the responsibility of removing the item from the XamDockManager control onto the ContentPane being removed which makes our life much easier.
        }

        /// <summary>
        /// Prepares a view being injected as a ContentPane
        /// </summary>
        /// <param name="item">the view</param>
        /// <returns>The injected view as a ContentPane</returns>
        protected virtual ContentPane PrepareContainerForItem(object item)
        {
            ContentPane container = item as ContentPane;

            if (container == null)
            {
                container = new ContentPane();
                container.Content = item; //the content is the view being injected
                container.DataContext = ResolveDataContext(item); //make sure the dataContext is the same as the view. Most likely a ViewModel
                container.SetValue(IsGeneratedProperty, true); //we generated this one
                CreateDockAwareBindings(container);
            }

            container.CloseAction = PaneCloseAction.RemovePane; //make it easy on ourselves and have the pane manage removing itself from the XamDockManager
            container.Closed += Container_Closed;

            return container;
        }

        void Container_Closed(object sender, Infragistics.Windows.DockManager.Events.PaneClosedEventArgs e)
        {
            ContentPane contentPane = sender as ContentPane;
            if (contentPane != null)
            {
                contentPane.Closed -= Container_Closed; //no memory leaks

                if (Region.Views.Contains(contentPane)) //this view was delcared in XAML and not injected
                    Region.Remove(contentPane);

                var item = contentPane.Content; //this view was injected and set as the content of our ContentPane
                if (item != null && Region.Views.Contains(item))
                    Region.Remove(item);

                ClearContainerForItem(contentPane); //reduce memory leaks
            }
        }

        /// <summary>
        /// Checks to see if the View or the View's DataContext (Most likely a ViewModel) implements the IDockAware interface and creates the necessary data bindings.
        /// </summary>
        /// <param name="container"></param>
        void CreateDockAwareBindings(ContentPane contentPane)
        {
            Binding binding = new Binding("Header");

            var dockAwareContainer = contentPane as IDockAware;
            if (dockAwareContainer != null)
            {
                binding.Source = dockAwareContainer;
            }

            var dockAwareDataContext = contentPane.DataContext as IDockAware;
            if (dockAwareDataContext != null)
            {
                binding.Source = dockAwareDataContext;
            }

            contentPane.SetBinding(ContentPane.HeaderProperty, binding);
        }

        /// <summary>
        /// Finds the DataContext of the view.
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        private object ResolveDataContext(object item)
        {
            FrameworkElement frameworkElement = item as FrameworkElement;
            return frameworkElement == null ? item : frameworkElement.DataContext;
        }

        /// <summary>
        /// Sets the Content property of a generated ContentPane to null.
        /// </summary>
        /// <param name="contentPane">The ContentPane</param>
        protected virtual void ClearContainerForItem(ContentPane contentPane)
        {
            if ((bool)contentPane.GetValue(IsGeneratedProperty))
                contentPane.Content = null;
        }
    }
}
